@@ -9,14 +9,13 @@ import pytest
 ptype = {'scipy': ScipyParticle, 'jit': JITParticle}
 
 
-def set_globcurrent_fieldset(filename=None, indices={}, full_load=False):
+def set_globcurrent_fieldset(filename=None, indices=None, full_load=False):
     if filename is None:
         filename = path.join(path.dirname(__file__), 'GlobCurrent_example_data',
                              '20*-GLOBCURRENT-L4-CUReul_hs-ALT_SUM-v02.0-fv01.0.nc')
-    filenames = {'U': filename, 'V': filename}
     variables = {'U': 'eastward_eulerian_current_velocity', 'V': 'northward_eulerian_current_velocity'}
     dimensions = {'lat': 'lat', 'lon': 'lon', 'time': 'time'}
-    return FieldSet.from_netcdf(filenames, variables, dimensions, indices, full_load=full_load)
+    return FieldSet.from_netcdf(filename, variables, dimensions, indices, full_load=full_load)
 
 
 def test_globcurrent_fieldset():
@@ -75,7 +74,23 @@ def test_globcurrent_particles(mode):
     assert(abs(pset[0].lat - -35.3) < 1)
 
 
-@pytest.mark.xfail(reason="Time extrapolation error expected to be thrown")
+def test__particles_init_time():
+    fieldset = set_globcurrent_fieldset()
+
+    lonstart = [25]
+    latstart = [-35]
+
+    # tests the different ways of initialising the time of a particle
+    pset = ParticleSet(fieldset, pclass=JITParticle, lon=lonstart, lat=latstart, time=np.datetime64('2002-01-15'))
+    pset2 = ParticleSet(fieldset, pclass=JITParticle, lon=lonstart, lat=latstart, time=14*86400)
+    pset3 = ParticleSet(fieldset, pclass=JITParticle, lon=lonstart, lat=latstart, time=np.array([np.datetime64('2002-01-15')]))
+    pset4 = ParticleSet(fieldset, pclass=JITParticle, lon=lonstart, lat=latstart, time=[np.datetime64('2002-01-15')])
+    assert pset[0].time - pset2[0].time == 0
+    assert pset[0].time - pset3[0].time == 0
+    assert pset[0].time - pset4[0].time == 0
+
+
+@pytest.mark.xfail(reason="Time extrapolation error expected to be thrown", strict=True)
 @pytest.mark.parametrize('mode', ['scipy', 'jit'])
 def test_globcurrent_time_extrapolation_error(mode):
     fieldset = set_globcurrent_fieldset()
@@ -97,3 +112,18 @@ def test_globcurrent_variable_fromfield(mode, dt):
     pset = ParticleSet(fieldset, pclass=MyParticle, lon=[25], lat=[-35], time=time)
 
     pset.execute(AdvectionRK4, runtime=delta(days=1), dt=dt)
+
+
+@pytest.mark.parametrize('full_load', [True, False])
+def test_globcurrent_deferred_fieldset_gradient(full_load):
+    fieldset = set_globcurrent_fieldset(full_load=full_load)
+    (dU_dx, dU_dy) = fieldset.U.gradient()
+    fieldset.add_field(dU_dy)
+
+    pset = ParticleSet(fieldset, pclass=JITParticle, lon=25, lat=-35)
+    pset.execute(AdvectionRK4, runtime=delta(days=1), dt=delta(days=1))
+
+    tdim = 365 if full_load else 3
+    assert(dU_dx.data.shape == (tdim, 41, 81))
+    assert(fieldset.dU_dy.data.shape == (tdim, 41, 81))
+    assert(dU_dx is fieldset.U.gradientx)
