@@ -10,7 +10,6 @@ class GridSet(object):
 
     def __init__(self):
         self.grids = []
-        self.size = 0
 
     def add_grid(self, field):
         grid = field.grid
@@ -33,5 +32,20 @@ class GridSet(object):
 
         if not existing_grid:
             self.grids.append(grid)
-            self.size += 1
         field.igrid = self.grids.index(field.grid)
+
+    def dimrange(self, dim):
+        """Returns maximum value of a dimension (lon, lat, depth or time)
+           on 'left' side and minimum value on 'right' side for all grids
+           in a gridset. Useful for finding e.g. longitude range that
+           overlaps on all grids in a gridset"""
+
+        maxleft, minright = (0, np.infty)
+        for g in self.grids:
+            maxleft = max(maxleft, getattr(g, dim)[0])
+            minright = min(minright, getattr(g, dim)[-1])
+        return maxleft, minright
+
+    @property
+    def size(self):
+        return len(self.grids)
